-- This set of two functions allow use of commas easier in Euphoria.
-- This code is placed in the public domain. Use it for whatever purpose
-- you like for free and at your own risk. (And please give me a little
-- credit! :)

-- 1998 Robert Brandon Pilkington

-----------
-- Usage --
-----------

-- s1 = add_commas(a, s2)
--
-- Returns atom a using printf/sprintf style fields for s2 in sequence s1.
-- Example:
--
-- puts(1, add_commas(-1234.5, ".2")
--
-- displays:
-- 
-- -1,234.50
--
-- Field widths are important. It counts the ENTIRE number including commas
-- negative sign, and decimal points. If you don't send at least ".0" you'll
-- get a decimal number in return. (the routine gets the sequence using a
-- floating point call to sprintf())

-- a = remove_commas(s)
--
-- Removes commas, dollar signs and percent signs from sequence s and returns
-- in atom a. a can be an integer as long as the number returned is an
-- integer. Very useful for getting string input allowing the user to use
-- commas and other stuff for a professional touch (and removing them to use
-- the number behind the user's back and adding them back with add_commas()
-- when displaying the data :)
--
-- Example:
--
-- printf(1, "%.2f", {remove_commas("$4,400,000.43")})
--
-- displays:
--
-- 4400000.43


include get.e

global function add_commas(atom number, sequence field)
    -- Adds commas to numbers to make them more readable.
    sequence revised, space
    atom float, modify
    space = ""
    modify = 1
    revised = sprintf("%" & field & "f", {number})
    while modify <= length(revised) do
        if revised[modify] = ' ' then
            space = space & ' '
            revised = revised[modify+1..length(revised)]
            modify = modify - 1
        end if
        modify = modify + 1
    end while
    float = find('.', revised)
    if float then
        for j = (float-4) to (find('-', revised) + 1) by -3 do
            revised = revised[1..j] & ',' & revised[j+1..length(revised)]
            if length(space) then space = space[1..length(space)-1] end if
        end for
    else
        if length(revised) > 3 then
            for j = (length(revised)-3) to (find('-', revised) + 1) by -3 do
                revised = revised[1..j] & ',' & revised[j+1..length(revised)]
                if length(space) then space = space[1..length(space)-1] end if
            end for
        end if
    end if
    return(space & revised)
end function

global function remove_commas(sequence num)
    -- Turn the sequence with commas and dollar signs into a usable atom
    object result
    atom c
    c = 0
    while c < length(num) do
        c = c + 1
        if find(num[c], "$,%") then
            num = num[1..c-1] & num[c+1..length(num)]
            c = c - 1
        end if
    end while
    result = value(num)
    if result[1] = GET_SUCCESS then
        result = result[2]
    else
        result = 0
    end if
    return result
end function

